#!/usr/bin/env node
"use strict";
/// <reference types="./typing" />
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.localStorage = exports.vorpal = void 0;
const vorpal_1 = __importDefault(require("@moleculer/vorpal"));
const ms_1 = __importDefault(require("ms"));
const utils_1 = require("./src/utils");
const queue_1 = require("./src/queue");
exports.vorpal = new vorpal_1.default();
exports.vorpal.localStorage("bull-repl-default");
exports.localStorage = exports.vorpal.localStorage;
exports.vorpal
    .command("connect <queue>", "Connect to bull queue")
    .option("--prefix <prefix>", "Prefix to use for all queue jobs")
    .option("-u, --uri <uri>", "Redis uri connection string")
    .option("-h, --host <host>", "Redis host for connection")
    .option("-p, --port <port>", "Redis port for connection")
    .option("-d, --db <db>", "Redis db for connection")
    .option("--password <password>", "Redis password for connection")
    .option("-c, --cert <cert>", "Absolute path to pem certificate if TLS used")
    .action(utils_1.wrapTryCatch(async (params) => {
    await queue_1.connectToQueue(params, exports.vorpal);
}));
exports.vorpal.command("connect-list", "List of saved connections").action(utils_1.wrapTryCatch(async () => {
    console.table(exports.localStorage._localStorage._keys);
}));
exports.vorpal.command("connect-rm <name>", "Remove saved connection").action(utils_1.wrapTryCatch(async ({ name }) => {
    if (name === utils_1.LAST_SAVED_CONNECTION_NAME) {
        return utils_1.logYellow(`Can't use reserved name, please use another`);
    }
    const savedItem = exports.localStorage.getItem(name);
    if (savedItem) {
        exports.localStorage.removeItem(name);
        utils_1.logGreen(`Connection "${name}" removed`);
    }
    else {
        utils_1.logYellow(`Connection "${name}" not found`);
    }
}));
exports.vorpal.command("connect-save <name>", "Save current connection").action(utils_1.wrapTryCatch(async ({ name: nameForSave }) => {
    if (nameForSave === utils_1.LAST_SAVED_CONNECTION_NAME) {
        return utils_1.logYellow(`Can't use reserved name, please use another`);
    }
    await queue_1.getQueue();
    const options = JSON.parse(exports.localStorage.getItem(utils_1.LAST_SAVED_CONNECTION_NAME));
    exports.localStorage.setItem(nameForSave, JSON.stringify(options));
    utils_1.logGreen(`Connection "${nameForSave}" saved`);
}));
exports.vorpal.command("connect-to <name>", "Connect to saved connection").action(utils_1.wrapTryCatch(async ({ name: connectToName }) => {
    const savedItem = exports.localStorage.getItem(connectToName);
    if (!savedItem) {
        return utils_1.logYellow(`Connection "${connectToName}" not found`);
    }
    const options = JSON.parse(savedItem);
    await queue_1.connectToQueue(options, exports.vorpal);
}));
exports.vorpal.command("stats", "Count of jobs by type").action(utils_1.wrapTryCatch(async () => {
    const queue = await queue_1.getQueue();
    const counts = await queue.getJobCounts("completed", "failed", "delayed", "repeat", "active", "wait", "paused");
    console.table(counts);
}));
exports.vorpal
    .command("active", "Fetch active jobs")
    .option("-q, --query <query>", `Query jobs via jq - ${utils_1.jqLink}. Notice, that bull data in root key e.g '[.root[] | select(.progress > 70)]'`)
    .option("-t, --timeAgo <timeAgo>", `Get jobs since time ago via ${utils_1.msLink}`)
    .option("-s, --start <start>", "Start index (pagination)")
    .option("-e, --end <end>", "End index (pagination)")
    .action(utils_1.wrapTryCatch(async ({ options }) => {
    const queue = await queue_1.getQueue();
    const timeAgoFilter = await utils_1.getTimeAgoFilter(options.timeAgo);
    utils_1.showJobs(await queue.getActive(options.start || 0, options.end || 100), [timeAgoFilter, options.query].filter(v => v).join(' | '));
}));
exports.vorpal
    .command("waiting", "Fetch waiting jobs")
    .option("-q, --query <query>", `Query jobs via jq - ${utils_1.jqLink}. Notice, that bull data in root key e.g '[.root[] | select(.progress > 70)]'`)
    .option("-t, --timeAgo <timeAgo>", `Get jobs since time ago via ${utils_1.msLink}`)
    .option("-s, --start <start>", "Start index (pagination)")
    .option("-e, --end <end>", "End index (pagination)")
    .action(utils_1.wrapTryCatch(async ({ options }) => {
    const queue = await queue_1.getQueue();
    const timeAgoFilter = await utils_1.getTimeAgoFilter(options.timeAgo);
    utils_1.showJobs(await queue.getWaiting(options.start || 0, options.end || 100), [timeAgoFilter, options.query].filter(v => v).join(' | '));
}));
exports.vorpal
    .command("completed", "Fetch completed jobs")
    .option("-q, --query <query>", `Query jobs via jq - ${utils_1.jqLink}. Notice, that bull data in root key e.g '[.root[] | select(.progress > 70)]'`)
    .option("-t, --timeAgo <timeAgo>", `Get jobs since time ago via ${utils_1.msLink}`)
    .option("-s, --start <start>", "Start index (pagination)")
    .option("-e, --end <end>", "End index (pagination)")
    .action(utils_1.wrapTryCatch(async ({ options }) => {
    const queue = await queue_1.getQueue();
    const timeAgoFilter = await utils_1.getTimeAgoFilter(options.timeAgo);
    utils_1.showJobs(await queue.getCompleted(options.start || 0, options.end || 100), [timeAgoFilter, options.query].filter(v => v).join(' | '));
}));
exports.vorpal
    .command("failed", "Fetch failed jobs")
    .option("-q, --query <query>", `Query jobs via jq - ${utils_1.jqLink}. Notice, that bull data in root key e.g '[.root[] | select(.progress > 70)]'`)
    .option("-t, --timeAgo <timeAgo>", `Get jobs since time ago via ${utils_1.msLink}`)
    .option("-s, --start <start>", "Start index (pagination)")
    .option("-e, --end <end>", "End index (pagination)")
    .action(utils_1.wrapTryCatch(async ({ options }) => {
    const queue = await queue_1.getQueue();
    const timeAgoFilter = await utils_1.getTimeAgoFilter(options.timeAgo);
    utils_1.showJobs(await queue.getFailed(options.start || 0, options.end || 100), [timeAgoFilter, options.query].filter(v => v).join(' | '));
}));
exports.vorpal
    .command("delayed", "Fetch delayed jobs")
    .option("-q, --query <query>", `Query jobs via jq - ${utils_1.jqLink}. Notice, that bull data in root key e.g '[.root[] | select(.progress > 70)]'`)
    .option("-t, --timeAgo <timeAgo>", `get jobs since time ago via ${utils_1.msLink}`)
    .option("-s, --start <start>", "start index (pagination)")
    .option("-e, --end <end>", "end index (pagination)")
    .action(utils_1.wrapTryCatch(async ({ options }) => {
    const queue = await queue_1.getQueue();
    const timeAgoFilter = await utils_1.getTimeAgoFilter(options.timeAgo);
    utils_1.showJobs(await queue.getDelayed(options.start || 0, options.end || 100), [timeAgoFilter, options.query].filter(v => v).join(' | '));
}));
exports.vorpal.command("pause", "Pause current queue").action(utils_1.wrapTryCatch(async () => {
    const queue = await queue_1.getQueue();
    await utils_1.answer(exports.vorpal, "Pause queue");
    await queue.pause();
    utils_1.logGreen(`Queue paused`);
}));
exports.vorpal.command("resume", "Resume current queue from pause").action(utils_1.wrapTryCatch(async () => {
    const queue = await queue_1.getQueue();
    await utils_1.answer(exports.vorpal, "Resume queue");
    await queue.resume();
    utils_1.logGreen(`Queue resumed from pause`);
}));
exports.vorpal.command("get <jobId...>", "Get job").action(utils_1.wrapTryCatch(async ({ jobId }) => {
    const { notFoundIds, foundJobs } = await utils_1.splitJobsByFound(jobId);
    notFoundIds.length && utils_1.logYellow(`Not found jobs: ${notFoundIds}`);
    foundJobs.length && utils_1.showJobs(foundJobs, '');
}));
exports.vorpal
    .command("add <data>", "Add job to queue e.g. add '{\"x\": 1}'")
    .option("-n, --name <name>", "name for named job")
    .action(utils_1.wrapTryCatch(async function ({ data, options }) {
    const queue = await queue_1.getQueue();
    let jobData;
    try {
        jobData = JSON.parse(data);
    }
    catch (e) {
        return utils_1.throwYellow(`Error: Argument <data> is invalid: ${e}`);
    }
    await utils_1.answer(exports.vorpal, "Add");
    const jobName = options.name || "__default__";
    const addedJob = await queue.add(jobName, jobData, {
        timestamp: Date.now()
    });
    utils_1.logGreen(`Job with name '${jobName}', id '${addedJob.id}' added`);
}));
exports.vorpal.command("rm <jobId...>", "Remove job").action(utils_1.wrapTryCatch(async function ({ jobId }) {
    await utils_1.answer(exports.vorpal, "Remove");
    const { notFoundIds, foundJobs } = await utils_1.splitJobsByFound(jobId);
    await Promise.all(foundJobs.map(j => j.remove()));
    notFoundIds.length && utils_1.logYellow(`Not found jobs: ${notFoundIds}`);
    foundJobs.length && utils_1.logGreen(`Jobs "${foundJobs.map(j => j.id)}" removed`);
}));
exports.vorpal.command("retry <jobId...>", "Retry job").action(utils_1.wrapTryCatch(async function ({ jobId }) {
    await utils_1.answer(exports.vorpal, "Retry");
    const { notFoundIds, foundJobs } = await utils_1.splitJobsByFound(jobId);
    await Promise.all(foundJobs.map(j => j.retry()));
    notFoundIds.length && utils_1.logYellow(`Not found jobs: ${notFoundIds}`);
    foundJobs.length && utils_1.logGreen(`Jobs "${foundJobs.map(j => j.id)}" retried`);
}));
exports.vorpal.command("retry-failed", "Retry first 100 failed jobs").action(utils_1.wrapTryCatch(async function () {
    const queue = await queue_1.getQueue();
    await utils_1.answer(exports.vorpal, "Retry failed jobs");
    const failedJobs = await queue.getFailed(0, 100);
    await Promise.all(failedJobs.map(j => j.retry()));
    utils_1.logGreen("All failed jobs retried");
}));
exports.vorpal.command("promote <jobId...>", "Promote job").action(utils_1.wrapTryCatch(async function ({ jobId }) {
    await utils_1.answer(exports.vorpal, "Promote");
    const { notFoundIds, foundJobs } = await utils_1.splitJobsByFound(jobId);
    await Promise.all(foundJobs.map(j => j.promote()));
    notFoundIds.length && utils_1.logYellow(`Not found jobs: ${notFoundIds}`);
    foundJobs.length && utils_1.logGreen(`Jobs "${foundJobs.map(j => j.id)}" promoted`);
}));
exports.vorpal.command("fail <jobId> <reason>", "Move job to failed").action(utils_1.wrapTryCatch(async function ({ jobId, reason }) {
    await queue_1.getQueue();
    const job = await utils_1.getJob(jobId);
    await utils_1.answer(exports.vorpal, "Fail");
    const err = new Error(reason);
    await job.moveToFailed(err, "0");
    utils_1.logGreen(`Job "${jobId}" failed`);
}));
exports.vorpal.command("complete <jobId> <data>", "Move job to completed e.g. complete 1 '{\"x\": 1}'")
    .action(utils_1.wrapTryCatch(async function ({ jobId, data }) {
    await queue_1.getQueue();
    const job = await utils_1.getJob(jobId);
    let returnValue;
    try {
        returnValue = JSON.parse(data);
    }
    catch (e) {
        return utils_1.throwYellow(`Error: Argument <data> is invalid: ${e}`);
    }
    await utils_1.answer(exports.vorpal, "Complete");
    await job.moveToCompleted(returnValue, "0");
    utils_1.logGreen(`Job "${jobId}" completed`);
}));
exports.vorpal
    .command("clean <period>", `Clean queue for period ago, period format - ${utils_1.msLink}`)
    .option("-s, --status <status>", "Status of the job to clean, default: completed")
    .option("-l, --limit <limit>", "Maximum amount of jobs to clean per call, default: all")
    .action(utils_1.wrapTryCatch(async function ({ period, options }) {
    const types = ["completed", "wait", "active", "delayed", "failed"];
    const queue = await queue_1.getQueue();
    const grace = period && period.length ? ms_1.default(period) : void 0;
    if (!grace) {
        return utils_1.throwYellow("Incorrect period");
    }
    const status = options.status || "completed";
    if (!types.includes(status)) {
        return utils_1.throwYellow(`Unknown status, must be one of: ${types.join(", ")}`);
    }
    await utils_1.answer(exports.vorpal, "Clean");
    const limit = Number.isInteger(options.limit)
        ? options.limit
        : 0;
    await queue.clean(grace, limit, status);
    utils_1.logGreen(`Jobs cleaned`);
}));
exports.vorpal
    .command("logs <jobId>", "Get logs of job")
    .option("-s, --start <start>", "Start of logs")
    .option("-e, --end <end>", "End of logs")
    .action(utils_1.wrapTryCatch(async ({ jobId, options }) => {
    const queue = await queue_1.getQueue();
    const { logs, count } = (await queue.getJobLogs(jobId, options.start, options.end));
    console.log(`Count of job logs: ${count}`);
    if (logs.length) {
        console.log("Logs:");
        utils_1.logArray(logs);
    }
}));
exports.vorpal.command("log <jobId> <data>", "Add log to job").action(utils_1.wrapTryCatch(async function ({ jobId, data }) {
    await queue_1.getQueue();
    const job = await utils_1.getJob(jobId);
    await utils_1.answer(exports.vorpal, "Add log");
    await job.log(data);
    utils_1.logGreen("Log added to job");
}));
exports.vorpal.command("events-on", "Turn on logging of queue events").action(utils_1.wrapTryCatch(async function () {
    const queue = await queue_1.getQueue();
    queue_1.listenQueueEvents(queue);
    utils_1.logGreen(`Logging of queue events enabled`);
}));
exports.vorpal.command("events-off", "Turn off logging of queue events").action(utils_1.wrapTryCatch(async function () {
    await queue_1.getQueue();
    queue_1.unlistenQueueEvents();
    utils_1.logGreen(`Logging of queue events disabled`);
}));
exports.vorpal.history("bull-repl-default");
const command = utils_1.getBootCommand(exports.vorpal.delimiter("BULL-REPL> ").show());
if (command) {
    exports.vorpal.exec(command);
}
